/* Datei: newton.c */
#include <stdio.h>
#include <math.h>

double f (double x)
{
   return (x - cos (x));
}

double fstrich (double x)
{
   return (1 + sin (x));
}

double berechne_nullstelle (double x, double genauigkeit,
                            double (*f) (double),
                            double (*fstrich) (double))
/* aequivalent hierzu ist:                                        */
/* double berechne_nullstelle (double x, double genauigkeit,      */
/*                             double f (double),                 */
/*                             double fstrich (double))           */
{
   double z = x;        /* Naeherungswert fuer die Nullstelle bei */
                        /* der Iteration                          */
   do
   {
      x = z;
      z = x - (*f)(x) / (*fstrich)(x);
      /* aequivalent hierzu ist:                                  */
      /* z = x - f(x) / fstrich(x);                               */
   } while (fabs (z - x) > genauigkeit);
   /* Die Funktion fabs() gibt den Betrag einer ihr uebergebenen  */
   /* Gleitpunktzahl zurueck.                                     */

   return z;
}

int main (void)
{
   double rohwert;     /* Rohwert fuer die Nullstelle             */
   double epsilon;     /* Genauigkeit der Nullstellenbestimmung   */

   printf ("\nGib den Rohwert fuer die Nullstelle ein: ");
   scanf ("%lf", &rohwert);

   printf ("\nGib den Wert fuer Epsilon ein: ");
   scanf ("%lf", &epsilon);

   printf ("\nDie Nullstelle ist bei: %10.2f\n",
            berechne_nullstelle (rohwert, epsilon, f, fstrich));
   return 0;
}





