/* Datei: conditionvar.c */
#include <stdio.h>
#include <threads.h>
#include <stdbool.h>

/* globale Variablen fuer Mutexe und Zustandsvariablen */
cnd_t queueNotEmpty, queueNotFull;
mtx_t queueMutex;

/* maximale Anzahl der Queue-Elemente */
#define QUEUE_SIZE 5

/* simulierte Queue (zaehlt den Queue-Inhalt) */
int elementZaehler = 0;

/* Terminierungsvariable (vereinfacht, nicht synchronisiert)*/
int terminate = false;

/* Produzent */
int producerThread (void * arg)
{
   (void) arg; /* wird nicht verwendet */
   while (true)
   {
      printf ("Erzeuge Daten\n");
      mtx_lock (&queueMutex); /* Sperren des Mutexes */

      /* pruefen, ob Queue voll ist */
      while (elementZaehler == QUEUE_SIZE) 
      {
         /* Warten auf das Eintreten des gewuenschten Zustands */
         cnd_wait (&queueNotFull, &queueMutex);
      }
        
      /* schreibe Daten in die Queue */
      elementZaehler++;
      cnd_broadcast (&queueNotEmpty);
      mtx_unlock (&queueMutex); /* Freigeben des Mutexes */

      if (terminate == true) /* Thread beenden? */
      {
         break;
      }
   }
   return thrd_success;
}

/* Konsument */
int consumerThread (void * arg)
{
   (void) arg; /* wird nicht verwendet */
   while (true)
   {
      printf ("Verbrauche Daten Thrd 0x%08x\n", thrd_current().p);
      mtx_lock (&queueMutex); /* Sperren des Mutexes */

      /* pruefen, ob Queue leer ist */
      while (elementZaehler == 0) 
      {
         /* Warten auf das Eintreten des gewuenschten Zustands */
         cnd_wait (&queueNotEmpty, &queueMutex);
      }
        
      /* lese Daten aus der Queue */
      elementZaehler--;
      cnd_broadcast (&queueNotFull);
      mtx_unlock (&queueMutex); /* Freigeben des Mutexes */

      if (terminate == true) /* Thread beenden? */
      {
         break;
      }
   }
   return thrd_success;
}

int main (void) 
{
   /* Variable zum Speichern der Thread ID */
   thrd_t producer, consumer1, consumer2;

   /* Initialisierung der Zustandsvariablen und des Mutexes */ 
   cnd_init (&queueNotFull); 
   cnd_init (&queueNotEmpty);
   mtx_init (&queueMutex, mtx_plain);


   /* Erzeugung der Threads */
   thrd_create (&producer, producerThread, NULL);
   thrd_create (&consumer1, consumerThread, NULL);
   thrd_create (&consumer2, consumerThread, NULL);

   for (int i = 0; i < 1e8; i++) 
   {
      /* eine Weile warten... */
   }

   /* Beenden der Threads */
   terminate = true;
   thrd_join (producer, NULL);
   thrd_join (consumer1, NULL);
   thrd_join (consumer2, NULL);

   /* Freigeben der Zustandsvariablen und des Mutexes */
   mtx_destroy (&queueMutex);
   cnd_destroy (&queueNotEmpty);
   cnd_destroy (&queueNotFull);
}